"""
Provides pydoctor version information.
"""

# This file is auto-generated! Do not edit!
# Use `python -m incremental.update pydoctor` to change this file.

from incremental import Version

__version__ = Version('pydoctor', 19, 11, 0)
__all__ = ["__version__"]
